@@ -128,7 +128,35 @@ void TransactionEditor::setup(QWidgetList& tabOrderWidgets, const MyMoneyAccount
   if (w)
     tabOrderWidgets.append(w);
   loadEditWidgets(action);
+
+  // remove all unused widgets and don't forget to remove them
+  // from the tab order list as well
   m_editWidgets.removeOrphans();
+  QWidgetList::iterator it_w;
+  const QWidgetList editWidgets(m_editWidgets.values());
+  for (it_w = tabOrderWidgets.begin(); it_w != tabOrderWidgets.end();) {
+    if (editWidgets.contains(*it_w)) {
+      ++it_w;
+    } else {
+      // before we remove the widget, we make sure it's not a part of a known one.
+      // these could be a direct child in case of KMyMoneyDateInput and KMyMoneyEdit
+      // where we store the pointer to the surrounding frame in editWidgets
+      // or the parent is called "KMyMoneyCategoryFrame"
+      if (*it_w) {
+        if (editWidgets.contains((*it_w)->parentWidget())
+            || ((*it_w)->parentWidget() && (*it_w)->parentWidget()->objectName() == QLatin1String("KMyMoneyCategoryFrame"))) {
+          ++it_w;
+
+        } else {
+          // qDebug("Remove '%s' from taborder", qPrintable((*it_w)->objectName()));
+          it_w = tabOrderWidgets.erase(it_w);
+        }
+      } else {
+        it_w = tabOrderWidgets.erase(it_w);
+      }
+    }
+  }
+
   clearFinalWidgets();
   setupFinalWidgets();
   slotUpdateButtonState();
@@ -171,48 +199,48 @@ bool TransactionEditor::eventFilter(QObject* o, QEvent* e)
       bool isFinal = false;
       QList<const QWidget*>::const_iterator it_w;
       switch (k->key()) {
-      case Qt::Key_Return:
-      case Qt::Key_Enter:
-        // we check, if the object is one of the m_finalEditWidgets and if it's
-        // a kMyMoneyEdit object that the value is not 0. If any of that is the
-        // case, it's the final object. In other cases, we convert the enter
-        // key into a TAB key to move between the fields. Of course, we only need
-        // to do this as long as the appropriate option is set. In all other cases,
-        // we treat the return/enter key as such.
-        if (KMyMoneyGlobalSettings::enterMovesBetweenFields()) {
-          for (it_w = m_finalEditWidgets.constBegin(); !isFinal && it_w != m_finalEditWidgets.constEnd(); ++it_w) {
-            if (*it_w == o) {
-              if (dynamic_cast<const kMyMoneyEdit*>(*it_w)) {
-                isFinal = !(dynamic_cast<const kMyMoneyEdit*>(*it_w)->value().isZero());
-              } else
-                isFinal = true;
+        case Qt::Key_Return:
+        case Qt::Key_Enter:
+          // we check, if the object is one of the m_finalEditWidgets and if it's
+          // a kMyMoneyEdit object that the value is not 0. If any of that is the
+          // case, it's the final object. In other cases, we convert the enter
+          // key into a TAB key to move between the fields. Of course, we only need
+          // to do this as long as the appropriate option is set. In all other cases,
+          // we treat the return/enter key as such.
+          if (KMyMoneyGlobalSettings::enterMovesBetweenFields()) {
+            for (it_w = m_finalEditWidgets.constBegin(); !isFinal && it_w != m_finalEditWidgets.constEnd(); ++it_w) {
+              if (*it_w == o) {
+                if (dynamic_cast<const kMyMoneyEdit*>(*it_w)) {
+                  isFinal = !(dynamic_cast<const kMyMoneyEdit*>(*it_w)->value().isZero());
+                } else
+                  isFinal = true;
+              }
             }
-          }
-        } else
-          isFinal = true;
+          } else
+            isFinal = true;
 
-        // for the non-final objects, we treat the return key as a TAB
-        if (!isFinal) {
-          QKeyEvent evt(e->type(),
-                        Qt::Key_Tab, k->modifiers(), QString(),
-                        k->isAutoRepeat(), k->count());
+          // for the non-final objects, we treat the return key as a TAB
+          if (!isFinal) {
+            QKeyEvent evt(e->type(),
+                          Qt::Key_Tab, k->modifiers(), QString(),
+                          k->isAutoRepeat(), k->count());
 
-          QApplication::sendEvent(o, &evt);
-          // in case of a category item and the split button is visible
-          // send a second event so that we get passed the button.
-          if (dynamic_cast<KMyMoneyCategory*>(o) && dynamic_cast<KMyMoneyCategory*>(o)->splitButton())
             QApplication::sendEvent(o, &evt);
+            // in case of a category item and the split button is visible
+            // send a second event so that we get passed the button.
+            if (dynamic_cast<KMyMoneyCategory*>(o) && dynamic_cast<KMyMoneyCategory*>(o)->splitButton())
+              QApplication::sendEvent(o, &evt);
 
-        } else {
-          QTimer::singleShot(0, this, SIGNAL(returnPressed()));
-        }
-        // don't process any further
-        rc = true;
-        break;
+          } else {
+            QTimer::singleShot(0, this, SIGNAL(returnPressed()));
+          }
+          // don't process any further
+          rc = true;
+          break;
 
-      case Qt::Key_Escape:
-        QTimer::singleShot(0, this, SIGNAL(escapePressed()));
-        break;
+        case Qt::Key_Escape:
+          QTimer::singleShot(0, this, SIGNAL(escapePressed()));
+          break;
       }
     }
   }
@@ -278,107 +306,107 @@ bool TransactionEditor::fixTransactionCommodity(const MyMoneyAccount& account)
     if (m_account.currencyId() != (*it_t).transaction().commodity()) {
       MyMoneySecurity osec = file->security((*it_t).transaction().commodity());
       switch ((*it_t).transaction().splitCount()) {
-      case 0:
-        // new transaction, guess nothing's here yet ;)
-        break;
+        case 0:
+          // new transaction, guess nothing's here yet ;)
+          break;
 
-      case 1:
-        try {
-          // make sure, that the value is equal to the shares, don't forget our own copy
-          MyMoneySplit& splitB = (*it_t).split();  // reference usage wanted here
-          if (m_split == splitB)
-            m_split.setValue(splitB.shares());
-          splitB.setValue(splitB.shares());
-          (*it_t).transaction().modifySplit(splitB);
+        case 1:
+          try {
+            // make sure, that the value is equal to the shares, don't forget our own copy
+            MyMoneySplit& splitB = (*it_t).split();  // reference usage wanted here
+            if (m_split == splitB)
+              m_split.setValue(splitB.shares());
+            splitB.setValue(splitB.shares());
+            (*it_t).transaction().modifySplit(splitB);
 
-        } catch (MyMoneyException *e) {
-          qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
-          delete e;
-        }
-        break;
-
-      case 2:
-        // If we deal with multiple currencies we make sure, that for
-        // transactions with two splits, the transaction's commodity is the
-        // currency of the currently selected account. This saves us from a
-        // lot of grieve later on.  We just have to switch the
-        // transactions commodity. Let's assume the following scenario:
-        // - transactions commodity is CA
-        // - splitB and account's currencyId is CB
-        // - splitA is of course in CA (otherwise we have a real problem)
-        // - Value is V in both splits
-        // - Shares in splitB is SB
-        // - Shares in splitA is SA (and equal to V)
-        //
-        // We do the following:
-        // - change transactions commodity to CB
-        // - set V in both splits to SB
-        // - modify the splits in the transaction
-        try {
-          // retrieve the splits
-          MyMoneySplit& splitB = (*it_t).split();  // reference usage wanted here
-          MyMoneySplit splitA = (*it_t).transaction().splitByAccount(m_account.id(), false);
-
-          // - set V in both splits to SB. Don't forget our own copy
-          if (m_split == splitB) {
-            m_split.setValue(splitB.shares());
+          } catch (MyMoneyException *e) {
+            qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
+            delete e;
           }
-          splitB.setValue(splitB.shares());
-          splitA.setValue(-splitB.shares());
-          (*it_t).transaction().modifySplit(splitA);
-          (*it_t).transaction().modifySplit(splitB);
+          break;
 
-        } catch (MyMoneyException *e) {
-          qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
-          delete e;
-        }
-        break;
+        case 2:
+          // If we deal with multiple currencies we make sure, that for
+          // transactions with two splits, the transaction's commodity is the
+          // currency of the currently selected account. This saves us from a
+          // lot of grieve later on.  We just have to switch the
+          // transactions commodity. Let's assume the following scenario:
+          // - transactions commodity is CA
+          // - splitB and account's currencyId is CB
+          // - splitA is of course in CA (otherwise we have a real problem)
+          // - Value is V in both splits
+          // - Shares in splitB is SB
+          // - Shares in splitA is SA (and equal to V)
+          //
+          // We do the following:
+          // - change transactions commodity to CB
+          // - set V in both splits to SB
+          // - modify the splits in the transaction
+          try {
+            // retrieve the splits
+            MyMoneySplit& splitB = (*it_t).split();  // reference usage wanted here
+            MyMoneySplit splitA = (*it_t).transaction().splitByAccount(m_account.id(), false);
 
-      default:
-        // TODO: use new logic by adjusting all splits by the price
-        // extracted from the selected split. Inform the user that
-        // this will happen and allow him to stop the processing (rc = false)
-
-        try {
-          QString msg;
-          if (firstTimeMultiCurrency) {
-            firstTimeMultiCurrency = false;
-            if (!isMultiSelection()) {
-              msg = i18n("This transaction has more than two splits and is originally based on a different currency (%1). Using this account to modify the transaction may result in rounding errors. Do you want to continue?", osec.name());
-            } else {
-              msg = i18n("At least one of the selected transactions has more than two splits and is originally based on a different currency (%1). Using this account to modify the transactions may result in rounding errors. Do you want to continue?", osec.name());
+            // - set V in both splits to SB. Don't forget our own copy
+            if (m_split == splitB) {
+              m_split.setValue(splitB.shares());
             }
+            splitB.setValue(splitB.shares());
+            splitA.setValue(-splitB.shares());
+            (*it_t).transaction().modifySplit(splitA);
+            (*it_t).transaction().modifySplit(splitB);
 
-            if (KMessageBox::warningContinueCancel(0, QString("<qt>%1</qt>").arg(msg)) == KMessageBox::Cancel) {
-              rc = false;
-            }
+          } catch (MyMoneyException *e) {
+            qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
+            delete e;
           }
+          break;
 
-          if (rc == true) {
-            MyMoneyMoney price;
-            if (!(*it_t).split().shares().isZero() && !(*it_t).split().value().isZero())
-              price = (*it_t).split().shares() / (*it_t).split().value();
-            QList<MyMoneySplit>::iterator it_s;
-            MyMoneySplit& mySplit = (*it_t).split();
-            for (it_s = (*it_t).transaction().splits().begin(); it_s != (*it_t).transaction().splits().end(); ++it_s) {
-              MyMoneySplit s = (*it_s);
-              if (s == mySplit) {
-                s.setValue(s.shares());
-                if (mySplit == m_split) {
-                  m_split = s;
-                }
-                mySplit = s;
+        default:
+          // TODO: use new logic by adjusting all splits by the price
+          // extracted from the selected split. Inform the user that
+          // this will happen and allow him to stop the processing (rc = false)
+
+          try {
+            QString msg;
+            if (firstTimeMultiCurrency) {
+              firstTimeMultiCurrency = false;
+              if (!isMultiSelection()) {
+                msg = i18n("This transaction has more than two splits and is originally based on a different currency (%1). Using this account to modify the transaction may result in rounding errors. Do you want to continue?", osec.name());
               } else {
-                s.setValue((s.value() * price).convert(fract));
+                msg = i18n("At least one of the selected transactions has more than two splits and is originally based on a different currency (%1). Using this account to modify the transactions may result in rounding errors. Do you want to continue?", osec.name());
+              }
+
+              if (KMessageBox::warningContinueCancel(0, QString("<qt>%1</qt>").arg(msg)) == KMessageBox::Cancel) {
+                rc = false;
               }
-              (*it_t).transaction().modifySplit(s);
             }
+
+            if (rc == true) {
+              MyMoneyMoney price;
+              if (!(*it_t).split().shares().isZero() && !(*it_t).split().value().isZero())
+                price = (*it_t).split().shares() / (*it_t).split().value();
+              QList<MyMoneySplit>::iterator it_s;
+              MyMoneySplit& mySplit = (*it_t).split();
+              for (it_s = (*it_t).transaction().splits().begin(); it_s != (*it_t).transaction().splits().end(); ++it_s) {
+                MyMoneySplit s = (*it_s);
+                if (s == mySplit) {
+                  s.setValue(s.shares());
+                  if (mySplit == m_split) {
+                    m_split = s;
+                  }
+                  mySplit = s;
+                } else {
+                  s.setValue((s.value() * price).convert(fract));
+                }
+                (*it_t).transaction().modifySplit(s);
+              }
+            }
+          } catch (MyMoneyException *e) {
+            qDebug("Unable to update commodity of split currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
+            delete e;
           }
-        } catch (MyMoneyException *e) {
-          qDebug("Unable to update commodity of split currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
-          delete e;
-        }
-        break;
+          break;
       }
 
       // set the transaction's ommodity to this account's currency
@@ -418,33 +446,33 @@ void TransactionEditor::setupCategoryWidget(KMyMoneyCategory* category, const QL
 #endif
 
   switch (splits.count()) {
-  case 0:
-    categoryId.clear();
-    if (!category->currentText().isEmpty()) {
-      category->clearEditText();
-      // make sure, we don't see the selector
-      category->completion()->hide();
-    }
-    category->completion()->setSelected(QString());
-    break;
+    case 0:
+      categoryId.clear();
+      if (!category->currentText().isEmpty()) {
+        category->clearEditText();
+        // make sure, we don't see the selector
+        category->completion()->hide();
+      }
+      category->completion()->setSelected(QString());
+      break;
 
-  case 1:
-    categoryId = splits[0].accountId();
-    category->completion()->setSelected(categoryId);
-    category->slotItemSelected(categoryId);
-    break;
+    case 1:
+      categoryId = splits[0].accountId();
+      category->completion()->setSelected(categoryId);
+      category->slotItemSelected(categoryId);
+      break;
 
-  default:
-    categoryId.clear();
-    category->setSplitTransaction();
-    connect(category, SIGNAL(focusIn()), this, splitEditSlot);
+    default:
+      categoryId.clear();
+      category->setSplitTransaction();
+      connect(category, SIGNAL(focusIn()), this, splitEditSlot);
 #if 0
-    // FIXME must deal with the logic that suppressObjectCreation is
-    // automatically turned off when the createItem() signal is connected
-    if (allowObjectCreation)
-      category->setSuppressObjectCreation(true);
+      // FIXME must deal with the logic that suppressObjectCreation is
+      // automatically turned off when the createItem() signal is connected
+      if (allowObjectCreation)
+        category->setSuppressObjectCreation(true);
 #endif
-    break;
+      break;
   }
 }
 
@@ -455,7 +483,7 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
 
   // make sure to run through all stuff that is tied to 'focusout events'.
   m_regForm->parentWidget()->setFocus();
-  QCoreApplication::processEvents(QEventLoop::ExcludeUserInput, 10);
+  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 10);
   // we don't need to update our widgets anymore, so we just disconnect the signal
   disconnect(file, SIGNAL(dataChanged()), this, SLOT(slotReloadEditWidgets()));
 
@@ -600,6 +628,25 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
         m_transactions.append(st);
       }
 
+      //    Save pricing information
+      QList<MyMoneySplit>::const_iterator it_t;
+      for (it_t = t.splits().constBegin(); it_t != t.splits().constEnd(); ++it_t) {
+        if (((*it_t).action() != "Buy") &&
+            ((*it_t).action() != "Sell") &&
+            ((*it_t).action() != "Reinvest")) {
+          continue;
+        }
+        QString id = (*it_t).accountId();
+        MyMoneyAccount acc = file->account(id);
+        MyMoneySecurity sec = file->security(acc.currencyId());
+        MyMoneyPrice price(acc.currencyId(),
+                           sec.tradingCurrency(),
+                           t.postDate(),
+                           (*it_t).price(), "Transaction");
+        file->addPrice(price);
+        break;
+      }
+
       ft.commit();
 
       // now analyze the balances and spit out warnings to the user
@@ -615,25 +662,25 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
           key = "minBalanceEarly";
           if (!acc.value(key).isEmpty()) {
             if (minBalanceEarly[acc.id()] == false && balance < MyMoneyMoney(acc.value(key))) {
-              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the warning balance of %2.", acc.name(), MyMoneyMoney(acc.value(key)).formatMoney(acc, sec)));
+              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the warning balance of %2.", acc.name(), MyMoneyUtils::formatMoney(MyMoneyMoney(acc.value(key)), acc, sec)));
             }
           }
           key = "minBalanceAbsolute";
           if (!acc.value(key).isEmpty()) {
             if (minBalanceAbsolute[acc.id()] == false && balance < MyMoneyMoney(acc.value(key))) {
-              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the minimum balance of %2.", acc.name(), MyMoneyMoney(acc.value(key)).formatMoney(acc, sec)));
+              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the minimum balance of %2.", acc.name(), MyMoneyUtils::formatMoney(MyMoneyMoney(acc.value(key)), acc, sec)));
             }
           }
           key = "maxCreditEarly";
           if (!acc.value(key).isEmpty()) {
             if (maxCreditEarly[acc.id()] == false && balance < MyMoneyMoney(acc.value(key))) {
-              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the maximum credit warning limit of %2.", acc.name(), MyMoneyMoney(acc.value(key)).formatMoney(acc, sec)));
+              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the maximum credit warning limit of %2.", acc.name(), MyMoneyUtils::formatMoney(MyMoneyMoney(acc.value(key)), acc, sec)));
             }
           }
           key = "maxCreditAbsolute";
           if (!acc.value(key).isEmpty()) {
             if (maxCreditAbsolute[acc.id()] == false && balance < MyMoneyMoney(acc.value(key))) {
-              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the maximum credit limit of %2.", acc.name(), MyMoneyMoney(acc.value(key)).formatMoney(acc, sec)));
+              msg = QString("<qt>%1</qt>").arg(i18n("The balance of account <b>%1</b> dropped below the maximum credit limit of %2.", acc.name(), MyMoneyUtils::formatMoney(MyMoneyMoney(acc.value(key)), acc, sec)));
             }
           }
 
@@ -655,6 +702,14 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
   return storeTransactions;
 }
 
+void TransactionEditor::resizeForm(void)
+{
+  // force resizeing of the columns in the form
+  KMyMoneyTransactionForm::TransactionForm* form = dynamic_cast<KMyMoneyTransactionForm::TransactionForm*>(m_regForm);
+  if (form) {
+    QMetaObject::invokeMethod(form, "resize", Qt::QueuedConnection, QGenericReturnArgument(), Q_ARG(int, ValueColumn1));
+  }
+}
 
 StdTransactionEditor::StdTransactionEditor()
 {
@@ -673,94 +728,111 @@ StdTransactionEditor::~StdTransactionEditor()
 
 void StdTransactionEditor::createEditWidgets(void)
 {
-  KMyMoneyCategory* account = new KMyMoneyCategory;
-  account->setClickMessage(i18n("Account"));
-  m_editWidgets["account"] = account;
-  connect(account, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateButtonState()));
-  connect(account, SIGNAL(itemSelected(const QString&)), this, SLOT(slotUpdateAccount(const QString&)));
+  // we only create the account widget in case it is needed
+  // to avoid confusion in the tab order later on.
+  if (m_item->showRowInForm(0)) {
+    KMyMoneyCategory* account = new KMyMoneyCategory;
+    account->setClickMessage(i18n("Account"));
+    account->setObjectName(QLatin1String("Account"));
+    m_editWidgets["account"] = account;
+    connect(account, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+    connect(account, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateAccount(QString)));
+  }
 
   KMyMoneyPayeeCombo* payee = new KMyMoneyPayeeCombo;
   payee->setClickMessage(i18n("Payer/Receiver"));
+  payee->setObjectName(QLatin1String("Payee"));
   m_editWidgets["payee"] = payee;
-  connect(payee, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateButtonState()));
-  connect(payee, SIGNAL(createItem(const QString&, QString&)), this, SIGNAL(createPayee(const QString&, QString&)));
+  connect(payee, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(payee, SIGNAL(createItem(QString,QString&)), this, SIGNAL(createPayee(QString,QString&)));
   connect(payee, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
-  connect(payee, SIGNAL(itemSelected(const QString&)), this, SLOT(slotUpdatePayee(const QString&)));
+  connect(payee, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdatePayee(QString)));
 
   KMyMoneyCategory* category = new KMyMoneyCategory(0, true);
   category->setClickMessage(i18n("Category/Account"));
+  category->setObjectName(QLatin1String("Category/Account"));
   m_editWidgets["category"] = category;
-  connect(category, SIGNAL(itemSelected(const QString&)), this, SLOT(slotUpdateCategory(const QString&)));
-  connect(category, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateButtonState()));
-  connect(category, SIGNAL(createItem(const QString&, QString&)), this, SLOT(slotCreateCategory(const QString&, QString&)));
+  connect(category, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateCategory(QString)));
+  connect(category, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(category, SIGNAL(createItem(QString,QString&)), this, SLOT(slotCreateCategory(QString,QString&)));
   connect(category, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
   connect(category->splitButton(), SIGNAL(clicked()), this, SLOT(slotEditSplits()));
-  category->splitButton()->setDisabled(true);
+  // initially disable the split button since we don't have an account set
+  if (category->splitButton())
+    category->splitButton()->setDisabled(m_account.id().isEmpty());
 
   KTextEdit* memo = new KTextEdit;
+  memo->setObjectName(QLatin1String("Memo"));
   memo->setTabChangesFocus(true);
   m_editWidgets["memo"] = memo;
 
   bool showNumberField = true;
   switch (m_account.accountType()) {
-  case MyMoneyAccount::Savings:
-  case MyMoneyAccount::Cash:
-  case MyMoneyAccount::Loan:
-  case MyMoneyAccount::AssetLoan:
-  case MyMoneyAccount::Asset:
-  case MyMoneyAccount::Liability:
-  case MyMoneyAccount::Equity:
-    showNumberField = KMyMoneyGlobalSettings::alwaysShowNrField();
-    break;
+    case MyMoneyAccount::Savings:
+    case MyMoneyAccount::Cash:
+    case MyMoneyAccount::Loan:
+    case MyMoneyAccount::AssetLoan:
+    case MyMoneyAccount::Asset:
+    case MyMoneyAccount::Liability:
+    case MyMoneyAccount::Equity:
+      showNumberField = KMyMoneyGlobalSettings::alwaysShowNrField();
+      break;
 
-  case MyMoneyAccount::Income:
-  case MyMoneyAccount::Expense:
-    showNumberField = false;
-    break;
+    case MyMoneyAccount::Income:
+    case MyMoneyAccount::Expense:
+      showNumberField = false;
+      break;
 
-  default:
-    break;
+    default:
+      break;
   }
 
   if (showNumberField) {
     kMyMoneyLineEdit* number = new kMyMoneyLineEdit;
     number->setClickMessage(i18n("Number"));
+    number->setObjectName(QLatin1String("Number"));
     m_editWidgets["number"] = number;
-    connect(number, SIGNAL(lineChanged(const QString&)), this, SLOT(slotNumberChanged(const QString&)));
+    connect(number, SIGNAL(lineChanged(QString)), this, SLOT(slotNumberChanged(QString)));
     // number->installEventFilter(this);
   }
 
   kMyMoneyDateInput* postDate = new kMyMoneyDateInput;
   m_editWidgets["postdate"] = postDate;
+  postDate->setObjectName(QLatin1String("PostDate"));
   postDate->setDate(QDate());
-  connect(postDate, SIGNAL(dateChanged(const QDate&)), this, SLOT(slotUpdateButtonState()));
+  connect(postDate, SIGNAL(dateChanged(QDate)), this, SLOT(slotUpdateButtonState()));
 
   kMyMoneyEdit* value = new kMyMoneyEdit;
   m_editWidgets["amount"] = value;
+  value->setObjectName(QLatin1String("Amount"));
   value->setResetButtonVisible(false);
-  connect(value, SIGNAL(valueChanged(const QString&)), this, SLOT(slotUpdateAmount(const QString&)));
-  connect(value, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateButtonState()));
+  connect(value, SIGNAL(valueChanged(QString)), this, SLOT(slotUpdateAmount(QString)));
+  connect(value, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
 
   value = new kMyMoneyEdit;
   m_editWidgets["payment"] = value;
+  value->setObjectName(QLatin1String("Payment"));
   value->setResetButtonVisible(false);
-  connect(value, SIGNAL(valueChanged(const QString&)), this, SLOT(slotUpdatePayment(const QString&)));
-  connect(value, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateButtonState()));
+  connect(value, SIGNAL(valueChanged(QString)), this, SLOT(slotUpdatePayment(QString)));
+  connect(value, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
 
   value = new kMyMoneyEdit;
   m_editWidgets["deposit"] = value;
+  value->setObjectName(QLatin1String("Deposit"));
   value->setResetButtonVisible(false);
-  connect(value, SIGNAL(valueChanged(const QString&)), this, SLOT(slotUpdateDeposit(const QString&)));
-  connect(value, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateButtonState()));
+  connect(value, SIGNAL(valueChanged(QString)), this, SLOT(slotUpdateDeposit(QString)));
+  connect(value, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
 
   KMyMoneyCashFlowCombo* cashflow = new KMyMoneyCashFlowCombo(0, m_account.accountGroup());
   m_editWidgets["cashflow"] = cashflow;
+  cashflow->setObjectName(QLatin1String("Cashflow"));
   connect(cashflow, SIGNAL(directionSelected(KMyMoneyRegister::CashFlowDirection)), this, SLOT(slotUpdateCashFlow(KMyMoneyRegister::CashFlowDirection)));
   connect(cashflow, SIGNAL(directionSelected(KMyMoneyRegister::CashFlowDirection)), this, SLOT(slotUpdateButtonState()));
 
   KMyMoneyReconcileCombo* reconcile = new KMyMoneyReconcileCombo;
   m_editWidgets["status"] = reconcile;
-  connect(reconcile, SIGNAL(itemSelected(const QString&)), this, SLOT(slotUpdateButtonState()));
+  reconcile->setObjectName(QLatin1String("Reconcile"));
+  connect(reconcile, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateButtonState()));
 
   KMyMoneyRegister::QWidgetContainer::iterator it_w;
   for (it_w = m_editWidgets.begin(); it_w != m_editWidgets.end(); ++it_w) {
@@ -800,8 +872,10 @@ void StdTransactionEditor::createEditWidgets(void)
   if (form) {
     KMyMoneyTransactionForm::TabBar* tabbar = new KMyMoneyTransactionForm::TabBar;
     m_editWidgets["tabbar"] = tabbar;
+    tabbar->setObjectName(QLatin1String("TabBar"));
     tabbar->copyTabs(form->tabBar());
     connect(tabbar, SIGNAL(tabCurrentChanged(int)), this, SLOT(slotUpdateAction(int)));
+    connect(tabbar, SIGNAL(tabCurrentChanged(int)), this, SIGNAL(operationTypeChanged(int)));
   }
 
   setupPrecision();
@@ -895,10 +969,11 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
 
     if ((w = haveWidget("number")) != 0) {
       dynamic_cast<kMyMoneyLineEdit*>(w)->loadText(m_split.number());
-      if (m_transaction.id().isEmpty()                           // new transaction
-          && dynamic_cast<kMyMoneyLineEdit*>(w)->text().isEmpty()    // no number filled in
-          && m_account.accountType() == MyMoneyAccount::Checkings    // checkings account
-          && KMyMoneyGlobalSettings::autoIncCheckNumber()) {         // and auto inc number turned on?
+      if (m_transaction.id().isEmpty()                              // new transaction
+          && dynamic_cast<kMyMoneyLineEdit*>(w)->text().isEmpty()   // no number filled in
+          && m_account.accountType() == MyMoneyAccount::Checkings   // checkings account
+          && KMyMoneyGlobalSettings::autoIncCheckNumber()           // and auto inc number turned on?
+          && action != KMyMoneyRegister::ActionDeposit) {           // only transfers or withdrawals
         assignNextNumber();
       }
     }
@@ -1027,8 +1102,9 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
 QWidget* StdTransactionEditor::firstWidget(void) const
 {
   QWidget* w = 0;
-  if (m_initialAction != KMyMoneyRegister::ActionNone)
+  if (m_initialAction != KMyMoneyRegister::ActionNone) {
     w = haveWidget("payee");
+  }
   return w;
 }
 
@@ -1325,19 +1401,20 @@ void StdTransactionEditor::slotUpdateAction(int action)
     QLabel* categoryLabel = dynamic_cast<QLabel*>(haveWidget("category-label"));
     KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(m_editWidgets["cashflow"]);
     switch (action) {
-    case KMyMoneyRegister::ActionDeposit:
-      categoryLabel->setText(i18n("Category"));
-      cashflow->setDirection(KMyMoneyRegister::Deposit);
-      break;
-    case KMyMoneyRegister::ActionTransfer:
-      categoryLabel->setText(i18n("Transfer from"));
-      slotUpdateCashFlow(cashflow->direction());
-      break;
-    case KMyMoneyRegister::ActionWithdrawal:
-      categoryLabel->setText(i18n("Category"));
-      cashflow->setDirection(KMyMoneyRegister::Payment);
-      break;
+      case KMyMoneyRegister::ActionDeposit:
+        categoryLabel->setText(i18n("Category"));
+        cashflow->setDirection(KMyMoneyRegister::Deposit);
+        break;
+      case KMyMoneyRegister::ActionTransfer:
+        categoryLabel->setText(i18n("Transfer from"));
+        slotUpdateCashFlow(cashflow->direction());
+        break;
+      case KMyMoneyRegister::ActionWithdrawal:
+        categoryLabel->setText(i18n("Category"));
+        cashflow->setDirection(KMyMoneyRegister::Payment);
+        break;
     }
+    resizeForm();
   }
 }
 
@@ -1355,6 +1432,7 @@ void StdTransactionEditor::slotUpdateCashFlow(KMyMoneyRegister::CashFlowDirectio
       } else {
         categoryLabel->setText(i18n("Transfer to"));
       }
+      resizeForm();
     } else {
       if (dir == KMyMoneyRegister::Deposit)
         tabbar->setCurrentIndex(KMyMoneyRegister::ActionDeposit);
@@ -1381,9 +1459,9 @@ void StdTransactionEditor::slotUpdateCategory(const QString& id)
     }
 
     if (tabbar) {
-      tabbar->widget(KMyMoneyRegister::ActionTransfer)->setEnabled(true);
-      tabbar->widget(KMyMoneyRegister::ActionDeposit)->setEnabled(true);
-      tabbar->widget(KMyMoneyRegister::ActionWithdrawal)->setEnabled(true);
+      tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, true);
+      tabbar->setTabEnabled(KMyMoneyRegister::ActionDeposit, true);
+      tabbar->setTabEnabled(KMyMoneyRegister::ActionWithdrawal, true);
     }
 
     if (!id.isEmpty()) {
@@ -1391,27 +1469,37 @@ void StdTransactionEditor::slotUpdateCategory(const QString& id)
       if (acc.isAssetLiability()
           || acc.accountGroup() == MyMoneyAccount::Equity) {
         if (tabbar) {
-          tabbar->widget(KMyMoneyRegister::ActionDeposit)->setEnabled(false);
-          tabbar->widget(KMyMoneyRegister::ActionWithdrawal)->setEnabled(false);
+          tabbar->setCurrentIndex(KMyMoneyRegister::ActionTransfer);
+          tabbar->setTabEnabled(KMyMoneyRegister::ActionDeposit, false);
+          tabbar->setTabEnabled(KMyMoneyRegister::ActionWithdrawal, false);
         }
-        if (val.isNegative())
+        if (val.isZero()) {
+          KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(m_editWidgets["cashflow"]);
+          if (cashflow && (cashflow->direction() == KMyMoneyRegister::Deposit)) {
+            categoryLabel->setText(i18n("Transfer from"));
+          } else {
+            categoryLabel->setText(i18n("Transfer to"));
+          }
+        } else if (val.isNegative())
           categoryLabel->setText(i18n("Transfer from"));
         else
           categoryLabel->setText(i18n("Transfer to"));
       } else {
         if (tabbar)
-          tabbar->widget(KMyMoneyRegister::ActionTransfer)->setEnabled(false);
+          tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
         categoryLabel->setText(i18n("Category"));
       }
       updateAmount(val.abs());
     } else {
       KMyMoneyCategory* category = dynamic_cast<KMyMoneyCategory*>(m_editWidgets["category"]);
       if (!category->currentText().isEmpty() && tabbar)
-        tabbar->widget(KMyMoneyRegister::ActionTransfer)->setEnabled(false);
+        tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
       categoryLabel->setText(i18n("Category"));
     }
     if (tabbar)
       tabbar->update();
+
+    resizeForm();
   }
   updateVAT(false);
 }
@@ -1692,13 +1780,16 @@ bool StdTransactionEditor::isComplete(QString& reason) const
     if (amount && !(amount->value().isZero()))
       break;
 
-    // the following two widgets are only checked if we are editing multiple transactions
+    // the following widgets are only checked if we are editing multiple transactions
     if (isMultiSelection()) {
       if (reconcile && reconcile->state() != MyMoneySplit::Unknown)
         break;
 
       if (cashflow && cashflow->direction() != KMyMoneyRegister::Unknown)
         break;
+
+      if (postDate->date().isValid() && (postDate->date() >= m_account.openingDate()))
+        break;
     }
   }
   return it_w != m_editWidgets.end();
@@ -1796,11 +1887,11 @@ int StdTransactionEditor::slotEditSplits(void)
                                  m_account,
                                  isValidAmount,
                                  dir == KMyMoneyRegister::Deposit,
-                                 MyMoneyMoney(0),
+                                 MyMoneyMoney(),
                                  m_priceInfo,
                                  m_regForm);
       connect(dlg, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
-      connect(dlg, SIGNAL(createCategory(MyMoneyAccount&, const MyMoneyAccount&)), this, SIGNAL(createCategory(MyMoneyAccount&, const MyMoneyAccount&)));
+      connect(dlg, SIGNAL(createCategory(MyMoneyAccount&,MyMoneyAccount)), this, SIGNAL(createCategory(MyMoneyAccount&,MyMoneyAccount)));
 
       if ((rc = dlg->exec()) == KDialog::Accepted) {
         m_transaction = dlg->transaction();
