@@ -44,6 +44,7 @@
 #include <kmymoneyplugin.h>
 #include <register.h>
 #include <kmymoneyutils.h>
+#include <mymoneyprice.h>
 
 class QResizeEvent;
 class QTreeWidgetItem;
@@ -59,7 +60,7 @@ class KPluginInfo;
  * read for new features that have been developed in KMyMoney.
  *
  * The latest version of this document is available from the project's web-site
- * at http://kmymoney2.sourceforge.net/ and is generated daily by doxygen reading
+ * at http://kmymoney.org/ and is generated daily by doxygen reading
  * the header files found in the CVS main branch.
  */
 
@@ -197,8 +198,6 @@ protected slots:
 
   void slotShowNextView(void);
 
-  void slotSecurityEditor(void);
-
   /**
     * Brings up a dialog to let the user search for specific transaction(s).  It then
     * opens a results window to display those transactions.
@@ -437,6 +436,8 @@ protected slots:
   void slotManageGpgKeys(void);
   void slotKeySelected(int idx);
 
+  void slotStatusProgressDone(void);
+
 public:
   /**
     * This method checks if there is at least one asset or liability account
@@ -670,6 +671,11 @@ protected:
    */
   void transactionMatch(void);
 
+  /**
+    * This method preloads the holidays for the duration of the default forecast period
+    */
+  void preloadHolidays(void);
+
 public slots:
   void slotFileInfoDialog(void);
 
@@ -912,6 +918,13 @@ public slots:
     */
   void slotShowCurrencyContextMenu(void);
 
+  /**
+    * This slot opens the price options menu at the current cursor
+    * position.
+    */
+  void slotShowPriceContextMenu(void);
+
+
   /**
     * This slot collects information for a new scheduled transaction
     * and saves it in the engine. @sa slotScheduleNew(const MyMoneyTransaction&)
@@ -963,6 +976,8 @@ public slots:
 
   void slotSelectCurrency(const MyMoneySecurity& currency = MyMoneySecurity());
 
+  void slotSelectPrice(const MyMoneyPrice& price = MyMoneyPrice());
+
   void slotTransactionMatch(void);
 
   /**
@@ -1055,18 +1070,19 @@ private:
     */
   void doDeleteTransactions(void);
 
-  /**
-    * Exchanges all references in transaction @a _t to account @a fromId
-    * into references to account @a toId. Returns @a true if at least
-    * one split has been changed, @a false otherwise.
-    */
-  bool exchangeAccountInTransaction(MyMoneyTransaction& _t, const QString& fromId, const QString& toId);
-
   /**
     * Re-implemented from IMyMoneyProcessingCalendar
     */
   bool isProcessingDate(const QDate& date) const;
 
+  /**
+    * Depending on the setting of AutoSaveOnQuit, this method
+    * asks the user to save the file or not.
+    *
+    * @returns see return values of KMessageBox::warningYesNoCancel()
+    */
+  int askSaveOnClose(void);
+
 signals:
   /**
     * This signal is emitted when a new file is loaded. In the case file
@@ -1138,12 +1154,24 @@ signals:
     */
   void currencySelected(const MyMoneySecurity& currency);
 
+  /**
+    * This signal is emitted when a new price has been selected by
+    * the GUI. If no price is selected or the selection is removed,
+    * @a price is identical to MyMoneyPrice().
+    */
+  void priceSelected(const MyMoneyPrice& price);
+
   void payeeRename(void);
   void payeeCreated(const QString& id);
 
   void currencyRename(void);
   void currencyCreated(const QString& id);
 
+  void priceEdit(void);
+  void priceNew(void);
+  void priceDelete(void);
+  void priceOnlineUpdate(void);
+
   void startMatchTransaction(const MyMoneyTransaction& t);
   void cancelMatchTransaction(void);
 
