@@ -67,6 +67,7 @@ class MyMoneyObjectContainer;
   * @li  Income
   * @li  Expense
   * @li  Loan (given)
+  * @li  Stock
   * @li  Equity
   *
   * @see MyMoneyInstitution
@@ -79,6 +80,8 @@ class MyMoneyObjectContainer;
 class KMM_MYMONEY_EXPORT MyMoneyAccount : public MyMoneyObject, public MyMoneyKeyValueContainer
 {
   friend class MyMoneyObjectContainer;
+  KMM_MYMONEY_UNIT_TESTABLE
+
 public:
 
   /**
@@ -570,6 +573,23 @@ public:
    */
   static QString accountTypeToString(const MyMoneyAccount::accountTypeE accountType);
 
+  /**
+    * keeps a history record of a reconciliation for this account on @a date
+    * with @a amount.
+    *
+    * @return @p true in case entry was added, @p false otherwise
+    *
+    * @sa reconciliationHistory()
+    */
+  bool addReconciliation(const QDate& date, const MyMoneyMoney& amount);
+
+  /**
+    * @return QMap with the reconciliation history for the account
+    *
+    * @sa addReconciliation()
+    */
+  const QMap<QDate, MyMoneyMoney>& reconciliationHistory();
+
   QDataStream &operator<<(const MyMoneyAccount &);
   QDataStream &operator>>(MyMoneyAccount &);
 
@@ -657,6 +677,10 @@ private:
     */
   int             m_fraction;
 
+  /**
+    * This member keeps the reconciliation history
+    */
+  QMap<QDate, MyMoneyMoney> m_reconciliationHistory;
 };
 
 /**
