@@ -24,6 +24,8 @@
 
 #include "querytable.h"
 
+#include <cmath>
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -64,7 +66,7 @@ QDate CashFlowListItem::m_sToday = QDate::currentDate();
 MyMoneyMoney CashFlowListItem::NPV(double _rate) const
 {
   double T = static_cast<double>(m_sToday.daysTo(m_date)) / 365.0;
-  MyMoneyMoney result(m_value.toDouble() / pow(1 + _rate, T));
+  MyMoneyMoney result(m_value.toDouble() / pow(1 + _rate, T), 100);
 
   //kDebug(2) << "CashFlowListItem::NPV( " << _rate << " ) == " << result;
 
@@ -92,7 +94,7 @@ CashFlowListItem CashFlowList::mostRecent(void) const
 
 MyMoneyMoney CashFlowList::NPV(double _rate) const
 {
-  MyMoneyMoney result(0.0);
+  MyMoneyMoney result;
 
   const_iterator it_cash = begin();
   while (it_cash != end()) {
@@ -166,7 +168,11 @@ double CashFlowList::xirrResult(double& rate) const
     double e_i = ((* list_it).today().daysTo((* list_it).date())) / 365.0;
     MyMoneyMoney val = (* list_it).value();
 
-    res += val.toDouble() / pow(r, e_i);
+    if (e_i < 0) {
+      res += val.toDouble() * pow(r, -e_i);
+    } else {
+      res += val.toDouble() / pow(r, e_i);
+    }
     ++list_it;
   }
 
@@ -253,93 +259,93 @@ QueryTable::QueryTable(const MyMoneyReport& _report): ListTable(_report)
 void QueryTable::init(void)
 {
   switch (m_config.rowType()) {
-  case MyMoneyReport::eAccountByTopAccount:
-  case MyMoneyReport::eEquityType:
-  case MyMoneyReport::eAccountType:
-  case MyMoneyReport::eInstitution:
-    constructAccountTable();
-    m_columns = "account";
-    break;
+    case MyMoneyReport::eAccountByTopAccount:
+    case MyMoneyReport::eEquityType:
+    case MyMoneyReport::eAccountType:
+    case MyMoneyReport::eInstitution:
+      constructAccountTable();
+      m_columns = "account";
+      break;
 
-  case MyMoneyReport::eAccount:
-    constructTransactionTable();
-    m_columns = "accountid,postdate";
-    break;
+    case MyMoneyReport::eAccount:
+      constructTransactionTable();
+      m_columns = "accountid,postdate";
+      break;
 
-  case MyMoneyReport::ePayee:
-  case MyMoneyReport::eMonth:
-  case MyMoneyReport::eWeek:
-    constructTransactionTable();
-    m_columns = "postdate,account";
-    break;
-  case MyMoneyReport::eCashFlow:
-    constructSplitsTable();
-    m_columns = "postdate";
-    break;
-  default:
-    constructTransactionTable();
-    m_columns = "postdate";
+    case MyMoneyReport::ePayee:
+    case MyMoneyReport::eMonth:
+    case MyMoneyReport::eWeek:
+      constructTransactionTable();
+      m_columns = "postdate,account";
+      break;
+    case MyMoneyReport::eCashFlow:
+      constructSplitsTable();
+      m_columns = "postdate";
+      break;
+    default:
+      constructTransactionTable();
+      m_columns = "postdate";
   }
 
   // Sort the data to match the report definition
   m_subtotal = "value";
 
   switch (m_config.rowType()) {
-  case MyMoneyReport::eCashFlow:
-    m_group = "categorytype,topcategory,category";
-    break;
-  case MyMoneyReport::eCategory:
-    m_group = "categorytype,topcategory,category";
-    break;
-  case MyMoneyReport::eTopCategory:
-    m_group = "categorytype,topcategory";
-    break;
-  case MyMoneyReport::eTopAccount:
-    m_group = "topaccount,account";
-    break;
-  case MyMoneyReport::eAccount:
-    m_group = "account";
-    break;
-  case MyMoneyReport::eAccountReconcile:
-    m_group = "account,reconcileflag";
-    break;
-  case MyMoneyReport::ePayee:
-    m_group = "payee";
-    break;
-  case MyMoneyReport::eMonth:
-    m_group = "month";
-    break;
-  case MyMoneyReport::eWeek:
-    m_group = "week";
-    break;
-  case MyMoneyReport::eAccountByTopAccount:
-    m_group = "topaccount";
-    break;
-  case MyMoneyReport::eEquityType:
-    m_group = "equitytype";
-    break;
-  case MyMoneyReport::eAccountType:
-    m_group = "type";
-    break;
-  case MyMoneyReport::eInstitution:
-    m_group = "institution,topaccount";
-    break;
-  default:
-    throw new MYMONEYEXCEPTION("QueryTable::QueryTable(): unhandled row type");
+    case MyMoneyReport::eCashFlow:
+      m_group = "categorytype,topcategory,category";
+      break;
+    case MyMoneyReport::eCategory:
+      m_group = "categorytype,topcategory,category";
+      break;
+    case MyMoneyReport::eTopCategory:
+      m_group = "categorytype,topcategory";
+      break;
+    case MyMoneyReport::eTopAccount:
+      m_group = "topaccount,account";
+      break;
+    case MyMoneyReport::eAccount:
+      m_group = "account";
+      break;
+    case MyMoneyReport::eAccountReconcile:
+      m_group = "account,reconcileflag";
+      break;
+    case MyMoneyReport::ePayee:
+      m_group = "payee";
+      break;
+    case MyMoneyReport::eMonth:
+      m_group = "month";
+      break;
+    case MyMoneyReport::eWeek:
+      m_group = "week";
+      break;
+    case MyMoneyReport::eAccountByTopAccount:
+      m_group = "topaccount";
+      break;
+    case MyMoneyReport::eEquityType:
+      m_group = "equitytype";
+      break;
+    case MyMoneyReport::eAccountType:
+      m_group = "type";
+      break;
+    case MyMoneyReport::eInstitution:
+      m_group = "institution,topaccount";
+      break;
+    default:
+      throw new MYMONEYEXCEPTION("QueryTable::QueryTable(): unhandled row type");
   }
 
   QString sort = m_group + ',' + m_columns + ",id,rank";
 
   switch (m_config.rowType()) {
-  case MyMoneyReport::eAccountByTopAccount:
-  case MyMoneyReport::eEquityType:
-  case MyMoneyReport::eAccountType:
-  case MyMoneyReport::eInstitution:
-    m_columns = "account";
-    break;
+    case MyMoneyReport::eAccountByTopAccount:
+    case MyMoneyReport::eEquityType:
+    case MyMoneyReport::eAccountType:
+    case MyMoneyReport::eInstitution:
+      m_columns = "account";
+      break;
 
-  default:
-    m_columns = "postdate";
+    default:
+      m_columns = "postdate";
   }
 
   unsigned qc = m_config.queryColumns();
@@ -392,22 +398,22 @@ void QueryTable::constructTransactionTable(void)
   bool hide_details;
 
   switch (m_config.rowType()) {
-  case MyMoneyReport::eCategory:
-  case MyMoneyReport::eTopCategory:
-    use_summary = false;
-    use_transfers = false;
-    hide_details = false;
-    break;
-  case MyMoneyReport::ePayee:
-    use_summary = false;
-    use_transfers = false;
-    hide_details = (m_config.detailLevel() == MyMoneyReport::eDetailNone);
-    break;
-  default:
-    use_summary = true;
-    use_transfers = true;
-    hide_details = (m_config.detailLevel() == MyMoneyReport::eDetailNone);
-    break;
+    case MyMoneyReport::eCategory:
+    case MyMoneyReport::eTopCategory:
+      use_summary = false;
+      use_transfers = false;
+      hide_details = false;
+      break;
+    case MyMoneyReport::ePayee:
+      use_summary = false;
+      use_transfers = false;
+      hide_details = (m_config.detailLevel() == MyMoneyReport::eDetailNone);
+      break;
+    default:
+      use_summary = true;
+      use_transfers = true;
+      hide_details = (m_config.detailLevel() == MyMoneyReport::eDetailNone);
+      break;
   }
 
   // support for opening and closing balances
@@ -446,7 +452,6 @@ void QueryTable::constructTransactionTable(void)
 
     const QList<MyMoneySplit>& splits = (*it_transaction).splits();
     QList<MyMoneySplit>::const_iterator myBegin, it_split;
-    //S_end = splits.end();
 
     for (it_split = splits.begin(), myBegin = splits.end(); it_split != splits.end(); ++it_split) {
       ReportAccount splitAcc = (* it_split).accountId();
@@ -470,6 +475,11 @@ void QueryTable::constructTransactionTable(void)
       myBegin = it_split;
     }
 
+    // skip this transaction if we didn't find a valid base account - see the above description
+    // for the base account's description - if we don't find it avoid a crash by skipping the transaction
+    if (myBegin == splits.end())
+      continue;
+
     // if the split is still unknown, use the first one. I have seen this
     // happen with a transaction that has only a single split referencing an income or expense
     // account and has an amount and value of 0. Such a transaction will fall through
@@ -764,17 +774,17 @@ void QueryTable::constructTransactionTable(void)
   // now run through our accts list and add opening and closing balances
 
   switch (m_config.rowType()) {
-  case MyMoneyReport::eAccount:
-  case MyMoneyReport::eTopAccount:
-    break;
+    case MyMoneyReport::eAccount:
+    case MyMoneyReport::eTopAccount:
+      break;
 
-    // case MyMoneyReport::eCategory:
-    // case MyMoneyReport::eTopCategory:
-    // case MyMoneyReport::ePayee:
-    // case MyMoneyReport::eMonth:
-    // case MyMoneyReport::eWeek:
-  default:
-    return;
+      // case MyMoneyReport::eCategory:
+      // case MyMoneyReport::eTopCategory:
+      // case MyMoneyReport::ePayee:
+      // case MyMoneyReport::eMonth:
+      // case MyMoneyReport::eWeek:
+    default:
+      return;
   }
 
   QDate startDate, endDate;
@@ -974,20 +984,16 @@ void QueryTable::constructPerformanceRow(const ReportAccount& account, TableRow&
       // find the split with the category, which has the actual amount of the dividend
       QList<MyMoneySplit> splits = (*it_transaction).splits();
       QList<MyMoneySplit>::const_iterator it_split = splits.constBegin();
-      bool found = false;
       while (it_split != splits.constEnd()) {
         ReportAccount acc = (*it_split).accountId();
         if (acc.isIncomeExpense()) {
-          found = true;
-          break;
+          cashincome += CashFlowListItem((*it_transaction).postDate(), -(*it_split).value() * price);
+          paidDividend += ((-(*it_split).value()) * price).convert(fraction);
         }
         ++it_split;
       }
-
-      if (found) {
-        cashincome += CashFlowListItem((*it_transaction).postDate(), -(*it_split).value() * price);
-        paidDividend += ((-(*it_split).value()) * price).convert(fraction);
-      }
+    } else if (action == MyMoneySplit::ActionAddShares) {
+      // Add shares is not a buy operation, do nothing
     } else {
       //if the split does not match any action above, add it as buy or sell depending on sign
 
@@ -1036,7 +1042,12 @@ void QueryTable::constructPerformanceRow(const ReportAccount& account, TableRow&
   }
 
   try {
-    MyMoneyMoney annualReturn = MyMoneyMoney(all.IRR(), 10000);
+    double irr = all.IRR();
+#ifdef Q_CC_MSVC
+    MyMoneyMoney annualReturn = MyMoneyMoney(_isnan(irr) ? 0 : irr, 10000);
+#else
+    MyMoneyMoney annualReturn = MyMoneyMoney(std::isnan(irr) ? 0 : irr, 10000);
+#endif
     result["return"] = annualReturn.toString();
     result["returninvestment"] = returnInvestment.toString();
   } catch (QString e) {
@@ -1084,7 +1095,7 @@ void QueryTable::constructAccountTable(void)
       // Handle currency conversion
       //
 
-      MyMoneyMoney displayprice(1.0);
+      MyMoneyMoney displayprice(1, 1);
       if (m_config.isConvertCurrency()) {
         // display currency is base currency, so set the price
         if (account.isForeignCurrency())
@@ -1376,17 +1387,17 @@ void QueryTable::constructSplitsTable(void)
   // now run through our accts list and add opening and closing balances
 
   switch (m_config.rowType()) {
-  case MyMoneyReport::eAccount:
-  case MyMoneyReport::eTopAccount:
-    break;
+    case MyMoneyReport::eAccount:
+    case MyMoneyReport::eTopAccount:
+      break;
 
-    // case MyMoneyReport::eCategory:
-    // case MyMoneyReport::eTopCategory:
-    // case MyMoneyReport::ePayee:
-    // case MyMoneyReport::eMonth:
-    // case MyMoneyReport::eWeek:
-  default:
-    return;
+      // case MyMoneyReport::eCategory:
+      // case MyMoneyReport::eTopCategory:
+      // case MyMoneyReport::ePayee:
+      // case MyMoneyReport::eMonth:
+      // case MyMoneyReport::eWeek:
+    default:
+      return;
   }
 
   QDate startDate, endDate;
